using System;
using System.Data;
using System.Configuration;
using System.Data.Common;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    public class AdminMenuPermissionDetails
    {
        private string _adminID;
        private string _menuPageName;
        private bool _viewMode;
        private bool _modifyMode;

        public string AdminID { get { return _adminID; } set { _adminID = value; } }
        public string MenuPageName { get { return _menuPageName; } set { _menuPageName = value; } }
        public bool ViewMode { get { return _viewMode; } set { _viewMode = value; } }
        public bool ModifyMode { get { return _modifyMode; } set { _modifyMode = value; } }
    }

    public class AdminMenuPermissionAccess
    {
        private const string AllPermissionFields = " AdminID, MenuPageName, ViewMode, ModifyMode ";

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "menupagename" ))
                return "ORDER BY M." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private static AdminMenuPermissionDetails ConvertDataTableToDetails( DataTable table )
        {
            AdminMenuPermissionDetails details = new AdminMenuPermissionDetails();
            if (table.Rows.Count > 0)
            {
                details.AdminID = table.Rows[0]["AdminID"].ToString();
                details.MenuPageName = table.Rows[0]["MenuPageName"].ToString();
                details.ViewMode = bool.Parse( table.Rows[0]["ViewMode"].ToString() );
                details.ModifyMode = bool.Parse( table.Rows[0]["ModifyMode"].ToString() );
            }
            return details;
        }

        public static AdminMenuPermissionDetails GetDetails( string adminID, string menuPageName )
        {
            DataTable result = DataAccess.ExecuteSelect(
                "SELECT " + AllPermissionFields + " " +
                "FROM AdminMenuPermission " +
                "WHERE AdminID = @AdminID AND MenuPageName = @MenuPageName; ",
                DataAccess.CreateParameterString( adminID ),
                DataAccess.CreateParameterString( menuPageName )
                );
            return ConvertDataTableToDetails( result );
        }

        public static DataTable GetAllByAdminID( string adminID )
        {
            DataTable result = DataAccess.ExecuteSelect(
                "SELECT " + AllPermissionFields + " " +
                "FROM AdminMenuPermission " +
                "WHERE AdminID = @AdminID; ",
                DataAccess.CreateParameterString( adminID )
                );
            return result;
        }

        public static bool IsExistingPermission( string adminID, string menuPageName )
        {
            DataTable dt = DataAccess.ExecuteSelect(
                " SELECT " + AllPermissionFields + " " +
                " FROM AdminMenuPermission " +
                " WHERE AdminID = @AdminID AND MenuPageName = @MenuPageName; ",
                DataAccess.CreateParameterString( adminID ),
                DataAccess.CreateParameterString( menuPageName )
            );

            if (dt.Rows.Count == 1)
                return true;
            else
                return false;
        }

        public static string Create(
            string adminID,
            string menuPageName,
            bool isViewMode,
            bool isModifyMode )
        {
            string resultID = DataAccess.ExecuteScalar(
                "INSERT INTO AdminMenuPermission ( AdminID , MenuPageName, ViewMode, ModifyMode ) " +
                "VALUES ( @AdminID, @MenuPageName, @ViewMode, @ModifyMode ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( adminID ),
                DataAccess.CreateParameterString( menuPageName ),
                DataAccess.CreateParameterBool( isViewMode ),
                DataAccess.CreateParameterBool( isModifyMode )
                );
            return resultID;
        }

        public static void Update(
            string adminID,
            string menuPageName,
            bool isViewMode,
            bool isModifyMode )
        {
            DataAccess.ExecuteNonQuery(
               "UPDATE AdminMenuPermission " +
               "SET " +
               "ViewMode = @ViewMode, ModifyMode = @ModifyMode " +
               "WHERE AdminID = @AdminID AND MenuPageName = @MenuPageName; ",
               DataAccess.CreateParameterBool( isViewMode ),
               DataAccess.CreateParameterBool( isModifyMode ),
               DataAccess.CreateParameterString( adminID ),
               DataAccess.CreateParameterString( menuPageName )
           );
        }

        public static void Delete(
            string adminID,
            string menuPageName
            )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM AdminMenuPermission " +
                "WHERE AdminID = @AdminID AND MenuPageName = @MenuPageName ",
                DataAccess.CreateParameterString( adminID ),
                DataAccess.CreateParameterString( menuPageName )
            );
        }

        public static void Delete(
            string adminID
            )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM AdminMenuPermission " +
                "WHERE AdminID = @AdminID  ",
                DataAccess.CreateParameterString( adminID )
            );
        }
    }
}
