using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.Utilities;
using Vevo.Shared.DataAccess;

namespace Vevo.Data
{
    public class ShippingOptionTypeRepositoryNoCache : IShippingOptionTypeRepository
    {
        #region Private

        private const string AllShippingOptionTypeFields =
            " ShippingOptionTypeID, TypeName, DisplayName, IsRealTime, RealTimeGatewayClassName, " +
            "ProviderClassName, AdminMainControl ";


        private ShippingOptionType CreateOneFromRow( DataRow row )
        {
            ShippingOptionType shippingOptionType = new ShippingOptionType();
            shippingOptionType.ShippingOptionTypeID = row["ShippingOptionTypeID"].ToString();
            shippingOptionType.TypeName = row["TypeName"].ToString();
            shippingOptionType.DisplayName = row["DisplayName"].ToString();
            shippingOptionType.IsRealTime = ConvertUtilities.ToBoolean( row["IsRealTime"] );
            shippingOptionType.RealTimeGatewayClassName = row["RealTimeGatewayClassName"].ToString();
            shippingOptionType.ProviderClassName = row["ProviderClassName"].ToString();
            shippingOptionType.AdminMainControl = row["AdminMainControl"].ToString();

            return shippingOptionType;
        }

        #endregion


        #region Public Methods

        public ShippingOptionType GetOne( string ShippingOptionTypeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingOptionTypeFields +
                "FROM ShippingOptionType " +
                "WHERE ShippingOptionTypeID = @ShippingOptionTypeID; ",
                DataAccess.CreateParameterString( ShippingOptionTypeID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return ShippingOptionType.Null;
        }

        public string GetIDByName( string typeName )
        {
            string id = DataAccess.ExecuteScalar(
                "SELECT ShippingOptionTypeID " +
                "FROM ShippingOptionType " +
                "WHERE TypeName = @TypeName; ",
                DataAccess.CreateParameterString( typeName ) );

            return ConvertUtilities.ToInt32( id ).ToString();
        }

        #endregion

    }
}
