using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for CustomerVerifyRequestState
    /// </summary>
    public class CustomerVerifyRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new CustomerVerifyResponseState( QBWCSession );
        }

        private void MoveToQueryState()
        {
            QBWCSession.CurrentState = new CustomerQueryRequestState( QBWCSession );
        }

        private IList<Customer> GetAllData()
        {
            IList<Customer> table = DataAccessContext.CustomerRepository.GetByIsQBExported( true, String.Empty );
            return table;
        }

        private CustomerQueryRqType CreateOneRequest( Customer customer )
        {
            CustomerQueryRqType req = new CustomerQueryRqType();
            req.Items = new object[] { customer.QBListID };
            req.ItemsElementName = new ItemsChoiceType25[] { ItemsChoiceType25.ListID };
            return req;
        }

        private CustomerQueryRqType CreateAnonymousRequest()
        {
            CustomerQueryRqType req = new CustomerQueryRqType();
            req.Items = new object[] { DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ) };
            req.ItemsElementName = new ItemsChoiceType25[] { ItemsChoiceType25.ListID };
            return req;
        }

        private string CreateCustomerVerify( out bool xmlReady )
        {
            xmlReady = false;

            IList<Customer> table = GetAllData();
            int lenght = table.Count;
            string qBAnonymousListID = DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ).Trim();
            if (qBAnonymousListID != "")
                lenght++;

            if (lenght == 0)
            {
                MoveToQueryState();
                return String.Empty;
            }

            QBXML qbxml = CreateRequestQbxml();

            object[] requests = new object[lenght];
            int index = 0;
            foreach (Customer customer in table)
            {
                requests[index] = CreateOneRequest( customer );
                index++;
            }

            if (qBAnonymousListID != "")
                requests[index] = CreateAnonymousRequest();

            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;

            return CreateRequestXml( qbxml );
        }

        public CustomerVerifyRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateCustomerVerify( out xmlReady );
        }
    }
}