using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.QuickBooks.Qbxml70;
using Vevo.Shared.Utilities;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for ProductModifyRequestState
    /// </summary>
    public class ProductModifyRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new ProductModifyResponseState( QBWCSession );
        }

        private void MoveAddRqState()
        {
            QBWCSession.CurrentState = new ProductAddRequestState( QBWCSession );
        }

        private IList<Product> GetModifyData()
        {
            IList<Product> table = new List<Product>();

            if (DataAccessContext.Configurations.GetBoolValue( "QBEnableProductModify" ))
            {
                string cultureID = DataAccessContext.CultureRepository.GetIDByName(
                DataAccessContext.Configurations.GetValue( "DefaultWebsiteLanguage" ) );
                Culture culture = DataAccessContext.CultureRepository.GetOne( cultureID );

                string storeID = getStoreID();
                string rootCategoryID = DataAccessContext.Configurations.GetValue( "0", "RootCategory", storeID );

                table = DataAccessContext.ProductRepository.GetByIsQBExported( culture, true, rootCategoryID, storeID, String.Empty );
            }

            return table;
        }

        private string getStoreID()
        {
            return DataAccessContext.Configurations.GetValue( "QBSelectStoreExport" );
        }

        private ItemNonInventoryModRqType CreateOneRequest( Product product, Dictionary<string, string> editSequenceList )
        {  
            string productName = CreateUniqueName( 
                QBUtilities.LimitLength( product.Name, QBConst.ProductNameMaxLength ),
                product.QBListID ); 
            
            ItemNonInventoryModRqType req = new ItemNonInventoryModRqType();
            req.ItemNonInventoryMod = new ItemNonInventoryMod();
            req.ItemNonInventoryMod.ListID = product.QBListID;
            req.ItemNonInventoryMod.EditSequence = editSequenceList[product.QBListID];
            req.ItemNonInventoryMod.Name = productName;

            SalesTaxCodeRef tax = new SalesTaxCodeRef();
            if (product.TaxClassID.Equals( "0" ))
                tax.FullName = "Non";
            else
                tax.FullName = "Tax";

            req.ItemNonInventoryMod.SalesTaxCodeRef = tax;
            req.ItemNonInventoryMod.ManufacturerPartNumber = product.ManufacturerPartNumber;

            SalesOrPurchaseMod sales = new SalesOrPurchaseMod();
            sales.Desc = QBUtilities.LimitLength( "ProductID: " + product.ProductID +
                " " + product.ShortDescription, QBConst.ProductDescMaxLength );
            sales.Item = ConvertUtilities.ToString( product.GetProductPrice( getStoreID() ).Price );
            sales.ItemElementName = ItemChoiceType3.Price;
            
            AccountRef accountRef = new AccountRef();
            accountRef.FullName = DataAccessContext.Configurations.GetValue( "QBIncomeAccountName" );
            sales.AccountRef = accountRef;
            req.ItemNonInventoryMod.Item = sales;
            return req;
        }

        private string CreateProductModify( out bool xmlReady )
        {
            IList<Product> table = GetModifyData();
            if (table.Count == 0)
            {
                xmlReady = false;
                MoveAddRqState();
                return String.Empty;
            }

            // Write QBXML
            QBXML qbxml = CreateRequestQbxml();
            object[] requests = new object[table.Count];
            int index = 0;
            Dictionary<string, string> editSequenceList = QBWCSession.GetEditSequenceList();

            foreach (Product product in table)
            {
                requests[index] = CreateOneRequest( product, editSequenceList );
                index++;
            }
            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;
            string xml = CreateRequestXml( qbxml );
            return xml;
        }

        public ProductModifyRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateProductModify( out xmlReady );
        }
    }
}