using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

using Vevo;
using Vevo.Domain;
using Vevo.WebAppLib;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Tester;
using Vevo.Shared.Utilities;

namespace Vevo.WebUI
{
    public class DatabaseConnectionService
    {
        #region Private

        private static bool _isConnectionOK = false;

        public static void BlankLogError( Exception ex )
        {
        }

        private string GetConnectionString()
        {
            return DataAccessHelper.CreateConnectionStringFromPrefix(
               WebConfiguration.DBConnectionStringPrefix,
               ConfigurationManager.ConnectionStrings[StoreContext.ConnectionStringsTagName].ConnectionString,
               ConfigurationManager.ConnectionStrings[StoreContext.ConnectionStringsTagName].ProviderName,
               HttpRuntime.AppDomainAppPath );
        }

        private bool TestConnection( out string errorMessage )
        {
            DatabaseTester tester = new DatabaseTester(
                GetConnectionString(),
                ConfigurationManager.ConnectionStrings[StoreContext.ConnectionStringsTagName].ProviderName );

            return tester.TestConnection( out errorMessage );
        }

        private bool IsRequestingUnitTestPath( HttpRequest request )
        {
            // Match the word "UnitTest" with no more subfolder under UnitTest
            Regex regex = new Regex( @"[/\\]unittest[/\\]?([^/\\]*)$", RegexOptions.IgnoreCase );

            return regex.IsMatch( request.Url.AbsolutePath );
        }

        private bool IsAtPage( HttpApplication app, string pageName )
        {
            return app.Request.Url.AbsolutePath.ToLower().Contains( pageName.ToLower() );
        }

        private bool TestConfiguration()
        {
            try
            {
                IList<Vevo.Domain.Configurations.Configuration> list = DataAccessContext.ConfigurationRepository.GetAll();
                return true;
            }
            catch
            {
                return false;
            }
        }

        #endregion


        #region Public Methods

        public bool TryConnect()
        {
            // Only use this database connecting method if the Payment App is "Standalone" web application.
            //if (!SiteContext.IsSeparatedWebApplication)
            //    return true;

            if (!_isConnectionOK)
            {
                DataAccessHelper.SetUp(
                    GetConnectionString(),
                    ConfigurationManager.ConnectionStrings[StoreContext.ConnectionStringsTagName].ProviderName,
                    true,
                    new DataAccess.LogErrorCallback( BlankLogError ),
                    WebConfiguration.SecretKey );

                HttpApplication app = HttpContext.Current.ApplicationInstance;

                string message;
                if (!TestConnection( out message )
                    && !IsAtPage( app, "SystemError.aspx" )
                    && !IsAtPage( app, "GenericError.aspx" )
                    && !IsRequestingUnitTestPath( app.Request ))
                {
                    return false;
                }

                _isConnectionOK = true;

                if (TestConfiguration())
                {
                    DataAccessContext.EnableConfigurations( true );
                    ConfigurationHelper.ApplyConfigurations();
                }
                else
                {
                    DataAccessContext.EnableConfigurations( false );
                }

                //PaymentAppInitializer initializer = new PaymentAppInitializer();
                //initializer.ProcessDatabaseConnected();
            }

            return true;
        }



        //public void RedirectToErrorPage()
        //{
        //    string errorHeader = "Database Connection Error";
        //    string errorText = "There is an error while attempting to connect to the database. Please verify your connection string in the file";
        //    HttpContext.Current.Response.Redirect( "~/SystemError.aspx" );
        //}

        #endregion
    }
}
