using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Vevo.Deluxe.Domain;
using Vevo.Deluxe.Domain.Products;
using Vevo.Deluxe.Domain.Users;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.WebUI.Orders;

namespace Vevo.WebUI.Products
{
    /// <summary>
    /// Summary description for BaseProductListItemUserControl
    /// </summary>
    public partial class BaseProductListItemUserControl : Vevo.WebUI.Products.BaseProductUserControl
    {
        private IMessageControl uxMessage
        {
            get
            {
                return (IMessageControl) FindControl( "uxMessage" );
            }
        }

        private bool NeedToRedirectToProductDetails( Product currentProduct )
        {
            if (currentProduct.ProductOptionGroups.Count > 0 &&
                !OptionVisibleInProductList( currentProduct.ProductID ))
                return true;
            else
                return false;
        }

        private bool CheckOptionList( Product currentProduct, out OptionItemValueCollection selectedOptions )
        {
            selectedOptions = null;
            bool validOption = false;
            if (currentProduct.ProductOptionGroups.Count > 0)
            {
                if (Page.IsValid && uxOptionGroupDetailsProxy.IsValidInput)
                {
                    selectedOptions = uxOptionGroupDetailsProxy.GetSelectedOptions();
                    validOption = true;
                }
                else
                    validOption = false;
            }
            else
                validOption = true;

            return validOption;
        }

        private void AddItemToShoppingCartWithoutOption( Product product, string urlName )
        {
            if (product.RequiresUserInput() || product.IsCustomPrice)
            {
                Response.Redirect( UrlManager.GetProductUrl( product.ProductID, urlName ) );
            }
            else
            {
                ProductSubscription subscription = new ProductSubscription( product.ProductID );
                StoreContext.ShoppingCart.AddItem( product, product.MinQuantity, subscription.IsSubscriptionProduct() );

                bool enableNotification = ConvertUtilities.ToBoolean( DataAccessContext.Configurations.GetValue( "EnableAddToCartNotification", StoreContext.CurrentStore ) );        
                if (UrlManager.IsMobileDevice(Request))
                {
                    enableNotification = false;
                }

                if (enableNotification)
                {
                    uxAddToCartNotificationProxy.Show( product, product.MinQuantity );
                }
                else
                {
                    Response.Redirect( "ShoppingCart.aspx" );
                }
            }
        }

        private void AddItemToShoppingCart( string productID, string urlName )
        {
            Product product = DataAccessContext.ProductRepository.GetOne(
                StoreContext.Culture, productID, new StoreRetriever().GetCurrentStoreID() );

            if (product.IsProductKit)
            {
                Response.Redirect( UrlManager.GetProductUrl( productID, urlName ) );
            }
            if (product.ProductOptionGroups.Count > 0)
            {
                if (!OptionVisibleInProductList( product.ProductID ))
                {
                    Response.Redirect( UrlManager.GetProductUrl( productID, urlName ) );
                }
                else
                {
                    if (uxOptionGroupDetailsProxy == null)
                    {
                        Response.Redirect( UrlManager.GetProductUrl( productID, urlName ) );
                    }
                    else if (Page.IsValid && uxOptionGroupDetailsProxy.IsValidInput)
                    {
                        OptionItemValueCollection selectedOptions = uxOptionGroupDetailsProxy.GetSelectedOptions();
                        CartItemGiftDetails giftDetails = new CartItemGiftDetails();

                        CartAddItemService addToCartService = new CartAddItemService(
                            StoreContext.Culture, StoreContext.ShoppingCart );
                        int currentStock;
                        string errorOptionName;
                        bool stockOK = addToCartService.AddToCart(
                            product,
                            selectedOptions,
                            ProductKitItemValueCollection.Null,
                            product.MinQuantity,
                            giftDetails,
                            0,
                            out errorOptionName,
                            out currentStock );

                        if (stockOK)
                        {
                            bool enableNotification = ConvertUtilities.ToBoolean( DataAccessContext.Configurations.GetValue( "EnableAddToCartNotification", StoreContext.CurrentStore ) );
                            if (UrlManager.IsMobileClient())
                            {
                                enableNotification = false;
                            }
                            if (enableNotification)
                            {
                                uxAddToCartNotificationProxy.Show( product, product.MinQuantity, 0, giftDetails, selectedOptions, ProductKitItemValueCollection.Null );
                            }
                            else
                            {
                                Response.Redirect( "ShoppingCart.aspx" );
                            }
                        }
                        else
                        {
                            DisplayOutOfStockError( currentStock, errorOptionName );
                        }
                    }
                }
            }
            else
            {
                AddItemToShoppingCartWithoutOption( product, urlName );
            }
        }

        protected bool OptionVisibleInProductList( string productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID, new StoreRetriever().GetCurrentStoreID() );

            if ((product.IsCustomPrice) || (product.IsProductKit))
                return false;
            else
                return DataAccessContext.Configurations.GetBoolValue( "IsDisplayOptionInProductList" );

        }

        protected void DisplayOutOfStockError( int currentStock, string name )
        {
            int amount = currentStock - DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" );

            if (amount < 0)
                amount = 0;

            string result = name + " " + amount;
            if (DataAccessContext.Configurations.GetBoolValue( "ShowQuantity" ))
            {
                uxMessage.DisplayError( String.Format(
                    "[$CheckStockMesssage1] {0} [$CheckStockMesssage2]", result.Trim() ) );
            }
            else
            {
                uxMessage.DisplayError( String.Format( "[$CheckStockMessage]" ) );
            }
        }

        protected void uxAddToCartImageButton_Command( object sender, EventArgs e )
        {
            LinkButton button = (LinkButton) sender;

            Product product = DataAccessContext.ProductRepository.GetOne(
                StoreContext.Culture, button.CommandArgument.ToString(), new StoreRetriever().GetCurrentStoreID() );

            ProductSubscription subscription = new ProductSubscription( product.ProductID );

            if (subscription.IsSubscriptionProduct())
            {
                if (StoreContext.Customer.IsNull)
                {
                    string returnUrl = "AddtoCart.aspx?ProductID=" + product.ProductID;
                    Response.Redirect( "~/UserLogin.aspx?ReturnUrl=" + returnUrl );
                }

                if (CustomerSubscription.IsContainsProductSubscriptionHigherLevel(
                subscription.ProductSubscriptions[0].SubscriptionLevelID,
                DataAccessContextDeluxe.CustomerSubscriptionRepository.GetCustomerSubscriptionsByCustomerID( StoreContext.Customer.CustomerID ) ))
                {
                    Response.Redirect( "AddShoppingCartNotComplete.aspx?ProductID=" + product.ProductID );
                }
            }

            if (!StoreContext.ShoppingCart.CheckCanAddItemToCart( product))
            {
                Response.Redirect( "AddShoppingCartNotComplete.aspx?ProductID=" + product.ProductID );
            }

            if (StoreContext.CheckoutDetails.ContainsGiftRegistry())
            {
                Response.Redirect( "AddShoppingCartNotComplete.aspx" );
            }
            else
            {
                AddItemToShoppingCart( button.CommandArgument.ToString(), button.CommandName.ToString() );
            }
        }

        protected bool QuantityDiscountVisible( object productDiscountGroupID, object categoryIDList )
        {
            return CatalogUtilities.GetQuantityDiscountByProductID( 
                productDiscountGroupID, categoryIDList );
        }

        protected void ProcessAddToWishListButton( IAddToWishListControl addToWishListControl, string productID )
        {
            Product currentProduct = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID, new StoreRetriever().GetCurrentStoreID() );

            if (NeedToRedirectToProductDetails( currentProduct ))
                Response.Redirect( UrlManager.GetProductUrl( currentProduct.ProductID, currentProduct.UrlName ) );

            OptionItemValueCollection selectedOptions;
            if (CheckOptionList( currentProduct, out selectedOptions ))
            {
                addToWishListControl.AddItemToWishListCart( currentProduct.ProductID, selectedOptions, currentProduct.MinQuantity.ToString() );
                Response.Redirect( "WishList.aspx" );
            }
        }

        public void Refresh()
        {
            uxOptionGroupDetailsProxy.PopulateControls();
        }

    }
}
