﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI.WebControls;
using Vevo.Domain.Products;
using Vevo.Domain;

namespace Vevo.WebUI.Products
{
    public class CategoryNavMenuBuilder
    {
        #region Private

        private Culture _culture;
        private GetCategoryUrlFunc _getCategoryUrl;
        private int _maxNodeDepth;

        private void PopulateCategories( int nodeDepth, MenuItem item, string parentCategoryID )
        {
            IList<Category> list = DataAccessContext.CategoryRepository.GetByParentID(
                _culture,
                parentCategoryID,
                "SortOrder",
                BoolFilter.ShowTrue );

            foreach (Category category in list)
            {
                item.ChildItems.Add( CreateMenuItemTree( nodeDepth, category ) );
            }
        }

        #endregion


        #region Constructors

        public CategoryNavMenuBuilder(
            Culture culture,
            GetCategoryUrlFunc getCategoryUrl,
            int maxNodeDepth )
        {
            _culture = culture;
            _getCategoryUrl = getCategoryUrl;
            _maxNodeDepth = maxNodeDepth;
        }

        #endregion


        #region Public Methods

        public delegate string GetCategoryUrlFunc( object categoryID, object urlName );

        public MenuItem CreateMenuItemTree( int nodeDepth, Category category )
        {
            MenuItem newItem = new MenuItem();
            newItem.Text = category.Name;
            newItem.Value = category.CategoryID;

            if (DataAccessContext.CategoryRepository.IsCategoryIDNotLeaf( category.CategoryID ))
            {
                newItem.NavigateUrl = _getCategoryUrl( category.CategoryID, category.UrlName );

                if (nodeDepth < _maxNodeDepth)
                    PopulateCategories( nodeDepth + 1, newItem, category.CategoryID );
            }
            else
            {
                newItem.NavigateUrl = _getCategoryUrl( category.CategoryID, category.UrlName );
            }
            return newItem;
        }

        #endregion

    }
}
